# frozen_string_literal: true

require "active_support/core_ext/numeric/bytes"
require "active_support/core_ext/numeric/time"
require "active_support/core_ext/numeric/inquiry"
require "active_support/core_ext/numeric/conversions"
