/*
  Fix a rendering issue affecting WebKits on Mac.
  See https://github.com/lifo/docrails/issues#issue/16 for more information.
*/
.syntaxhighlighter a,
.syntaxhighlighter div,
.syntaxhighlighter code,
.syntaxhighlighter table,
.syntaxhighlighter table td,
.syntaxhighlighter table tr,
.syntaxhighlighter table tbody,
.syntaxhighlighter table thead,
.syntaxhighlighter table caption,
.syntaxhighlighter textarea {
  line-height: 1.25em !important;
}
